// A program to determine the minimum number of coins needed to make change.
// (c) 1997, 2001 duane a. bailey
import structure.*;

public class FullPostage
{
    static int count1, count2;
    public static final int LETTER = 37; // letter rate
    public static final int CARD = 21;   // post card rate
    public static final int PENNY = 1;   // penny stamp

    public static int stampCount(int amount)
    // pre: amount >= 0
    // post: return *number* of stamps needed to make change
    //       (only use letter, post card, and penny stamps)
    {
	return stampCount(amount, new int[amount+1]);
    }

    protected static int stampCount(int amount, int answer[])
    // pre: amount >= 0; answer array has length >= amount
    // post: return *number* of stamps needed to make change
    //       (only use letter, post card, and penny stamps)
    {
	int minStamps;
	count1++;
	Assert.pre(amount >= 0,"Reasonable amount of change.");
	if (amount == 0) return 0;
	if (answer[amount] != 0) return answer[amount];
	// consider use of a penny stamp
	minStamps = 1+stampCount(amount-1,answer);
	// consider use of a post card stamp
	if (amount >= CARD) {
	    int possible = 1+stampCount(amount-CARD,answer);
	    if (minStamps > possible) minStamps = possible;
	}
	// consider use of a letter stamp
	if (amount >= LETTER) {
	    int possible = 1+stampCount(amount-LETTER,answer);
	    if (minStamps > possible) minStamps = possible;
	}
	answer[amount] = minStamps;
	return minStamps;
    } 

    public static void main(String args[])
    {
	for (int amount = 1; amount < 100; amount++)
	{
	    count1 = 0;
	    System.out.println("Stamps returned to make "+amount+" cents: "+
			       stampCount(amount)+" ("+count1+" calls)");
	}
    }

    static Vector amounts;
    public static coins change(int amount)
    {
	Assert.pre(amount >= 0, "Amount must be positive.");
	amounts.ensureCapacity(amount+1);
	if (amounts.get(amount) == null)
	{	
	    coins best, possible;
	    if (amount == 0) best = new coins();
	    else {
		possible = change(amount-1).addPenny();
		best = possible;
		if (amount >= 5) {
		    possible = change(amount-5).addNickle();
		    if (best.total() > possible.total()) best = possible;
		}
		if (amount >= 10) {
		    possible = change(amount-10).addDime();
		    if (best.total() > possible.total()) best = possible;
		}
		if (amount >= 25) {
		    possible = change(amount-25).addQuarter();
		    if (best.total() > possible.total()) best = possible;
		}
	    }
	    amounts.set(amount,best);
	}
	return new coins((coins)amounts.get(amount));
    } 

    /*    public static void main(String args[]) {
	amounts = new Vector();
	for (int i = 0; i < 100; i++)
	{
	    System.out.println("Change for "+i+" cents: "+change(i));
	}
    }*/

}

class coins
{
    int pennies, nickles, dimes, quarters;
    public coins()
    {
	pennies = nickles = dimes = quarters = 0;
    }
    public coins(coins that)
    {
	this.pennies = that.pennies;
	this.nickles = that.nickles;
	this.dimes = that.dimes;
	this.quarters = that.quarters;
    }
    public coins addPenny() { pennies++; return this;}
    public coins addNickle() { nickles++; return this;}
    public coins addDime() { dimes++; return this;}
    public coins addQuarter() { quarters++; return this;}
    public int total() { return pennies+nickles+dimes+quarters; }
    public String toString() {
	return "pennies: "+pennies+", nickles: "+nickles+
	       ", dimes: "+dimes+", quarters: "+quarters;
    }
}

/*
Stamps returned to make 1 cents: 1 (2 calls)
Stamps returned to make 2 cents: 2 (3 calls)
Stamps returned to make 3 cents: 3 (4 calls)
Stamps returned to make 4 cents: 4 (5 calls)
Stamps returned to make 5 cents: 5 (6 calls)
Stamps returned to make 6 cents: 6 (7 calls)
Stamps returned to make 7 cents: 7 (8 calls)
Stamps returned to make 8 cents: 8 (9 calls)
Stamps returned to make 9 cents: 9 (10 calls)
Stamps returned to make 10 cents: 10 (11 calls)
Stamps returned to make 11 cents: 11 (12 calls)
Stamps returned to make 12 cents: 12 (13 calls)
Stamps returned to make 13 cents: 13 (14 calls)
Stamps returned to make 14 cents: 14 (15 calls)
Stamps returned to make 15 cents: 15 (16 calls)
Stamps returned to make 16 cents: 16 (17 calls)
Stamps returned to make 17 cents: 17 (18 calls)
Stamps returned to make 18 cents: 18 (19 calls)
Stamps returned to make 19 cents: 19 (20 calls)
Stamps returned to make 20 cents: 20 (21 calls)
Stamps returned to make 21 cents: 1 (23 calls)
Stamps returned to make 22 cents: 2 (25 calls)
Stamps returned to make 23 cents: 3 (27 calls)
Stamps returned to make 24 cents: 4 (29 calls)
Stamps returned to make 25 cents: 5 (31 calls)
Stamps returned to make 26 cents: 6 (33 calls)
Stamps returned to make 27 cents: 7 (35 calls)
Stamps returned to make 28 cents: 8 (37 calls)
Stamps returned to make 29 cents: 9 (39 calls)
Stamps returned to make 30 cents: 10 (41 calls)
Stamps returned to make 31 cents: 11 (43 calls)
Stamps returned to make 32 cents: 12 (45 calls)
Stamps returned to make 33 cents: 13 (47 calls)
Stamps returned to make 34 cents: 14 (49 calls)
Stamps returned to make 35 cents: 15 (51 calls)
Stamps returned to make 36 cents: 16 (53 calls)
Stamps returned to make 37 cents: 1 (56 calls)
Stamps returned to make 38 cents: 2 (59 calls)
Stamps returned to make 39 cents: 3 (62 calls)
Stamps returned to make 40 cents: 4 (65 calls)
Stamps returned to make 41 cents: 5 (68 calls)
Stamps returned to make 42 cents: 2 (71 calls)
Stamps returned to make 43 cents: 3 (74 calls)
Stamps returned to make 44 cents: 4 (77 calls)
Stamps returned to make 45 cents: 5 (80 calls)
Stamps returned to make 46 cents: 6 (83 calls)
Stamps returned to make 47 cents: 7 (86 calls)
Stamps returned to make 48 cents: 8 (89 calls)
Stamps returned to make 49 cents: 9 (92 calls)
Stamps returned to make 50 cents: 10 (95 calls)
Stamps returned to make 51 cents: 11 (98 calls)
Stamps returned to make 52 cents: 12 (101 calls)
Stamps returned to make 53 cents: 13 (104 calls)
Stamps returned to make 54 cents: 14 (107 calls)
Stamps returned to make 55 cents: 15 (110 calls)
Stamps returned to make 56 cents: 16 (113 calls)
Stamps returned to make 57 cents: 17 (116 calls)
Stamps returned to make 58 cents: 2 (119 calls)
Stamps returned to make 59 cents: 3 (122 calls)
Stamps returned to make 60 cents: 4 (125 calls)
Stamps returned to make 61 cents: 5 (128 calls)
Stamps returned to make 62 cents: 6 (131 calls)
Stamps returned to make 63 cents: 3 (134 calls)
Stamps returned to make 64 cents: 4 (137 calls)
Stamps returned to make 65 cents: 5 (140 calls)
Stamps returned to make 66 cents: 6 (143 calls)
Stamps returned to make 67 cents: 7 (146 calls)
Stamps returned to make 68 cents: 8 (149 calls)
Stamps returned to make 69 cents: 9 (152 calls)
Stamps returned to make 70 cents: 10 (155 calls)
Stamps returned to make 71 cents: 11 (158 calls)
Stamps returned to make 72 cents: 12 (161 calls)
Stamps returned to make 73 cents: 13 (164 calls)
Stamps returned to make 74 cents: 2 (167 calls)
Stamps returned to make 75 cents: 3 (170 calls)
Stamps returned to make 76 cents: 4 (173 calls)
Stamps returned to make 77 cents: 5 (176 calls)
Stamps returned to make 78 cents: 6 (179 calls)
Stamps returned to make 79 cents: 3 (182 calls)
Stamps returned to make 80 cents: 4 (185 calls)
Stamps returned to make 81 cents: 5 (188 calls)
Stamps returned to make 82 cents: 6 (191 calls)
Stamps returned to make 83 cents: 7 (194 calls)
Stamps returned to make 84 cents: 4 (197 calls)
Stamps returned to make 85 cents: 5 (200 calls)
Stamps returned to make 86 cents: 6 (203 calls)
Stamps returned to make 87 cents: 7 (206 calls)
Stamps returned to make 88 cents: 8 (209 calls)
Stamps returned to make 89 cents: 9 (212 calls)
Stamps returned to make 90 cents: 10 (215 calls)
Stamps returned to make 91 cents: 11 (218 calls)
Stamps returned to make 92 cents: 12 (221 calls)
Stamps returned to make 93 cents: 13 (224 calls)
Stamps returned to make 94 cents: 14 (227 calls)
Stamps returned to make 95 cents: 3 (230 calls)
Stamps returned to make 96 cents: 4 (233 calls)
Stamps returned to make 97 cents: 5 (236 calls)
Stamps returned to make 98 cents: 6 (239 calls)
Stamps returned to make 99 cents: 7 (242 calls)
*/
