// A program to determine the minimum number of coins needed to make change.
// (c) 1997, 2001 duane a. bailey
import structure.*;

public class RecursivePostage
{
    static int count1;
    public final static int LETTER=37;
    public final static int CARD=21;
    public final static int PENNY=1;

    public static int stampCount(int amount)
    // pre: amount >= 0
    // post: return *number* of stamps needed to make change
    //       (only use letter, card, and penny stamps)
    {
	int minStamps;
	count1++;
	Assert.pre(amount >= 0,"Reasonable amount of change.");
	if (amount == 0) return 0;
	// consider use of a penny stamp
	minStamps = 1+stampCount(amount-PENNY);
	// consider use of a post card stamp
	if (amount >= CARD) {
	    int possible = 1+stampCount(amount-CARD);
	    if (minStamps > possible) minStamps = possible;
	}
	// consider use of a letter stamp
	if (amount >= LETTER) {
	    int possible = 1+stampCount(amount-LETTER);
	    if (minStamps > possible) minStamps = possible;
	}
	return minStamps;
    } 

    public static void main(String args[])
    {
	for (int amount = 1; amount < 100; amount++)
	{
	    count1 = 0;
	    System.out.println("Stamps returned to make "+amount+" cents: "+
			       stampCount(amount)+" ("+count1+" calls)");
	}
    }

    public static coins change(int amount)
    {
	Assert.pre(amount >= 0, "Amount must be positive.");
	coins best, possible;
	if (amount == 0) best = new coins();
	else {
	    possible = change(amount-1);
	    possible.addPenny();
	    best = possible;
	    if (amount >= 5) {
		possible = change(amount-5);
		possible.addNickle();
		if (best.total() > possible.total()) best = possible;
	    }
	    if (amount >= 10) {
		possible = change(amount-10);
		possible.addDime();
		if (best.total() > possible.total()) best = possible;
	    }
	    if (amount >= 25) {
		possible = change(amount-25);
		possible.addQuarter();
		if (best.total() > possible.total()) best = possible;
	    }
	}
	return new coins(best);
    } 
}

class coins
{
    int pennies, nickles, dimes, quarters;
    public coins()
    {
	pennies = nickles = dimes = quarters = 0;
    }
    public coins(coins that)
    {
	this.pennies = that.pennies;
	this.nickles = that.nickles;
	this.dimes = that.dimes;
	this.quarters = that.quarters;
    }
    public void addPenny() { pennies++; }
    public void addNickle() { nickles++; }
    public void addDime() { dimes++; }
    public void addQuarter() { quarters++; }
    public int total() { return pennies+nickles+dimes+quarters; }
    public String toString() {
	return "pennies: "+pennies+", nickles: "+nickles+
	       ", dimes: "+dimes+", quarters: "+quarters;
    }
}

/*
Stamps returned to make 1 cents: 1 (2 calls)
Stamps returned to make 2 cents: 2 (3 calls)
Stamps returned to make 3 cents: 3 (4 calls)
Stamps returned to make 4 cents: 4 (5 calls)
Stamps returned to make 5 cents: 5 (6 calls)
Stamps returned to make 6 cents: 6 (7 calls)
Stamps returned to make 7 cents: 7 (8 calls)
Stamps returned to make 8 cents: 8 (9 calls)
Stamps returned to make 9 cents: 9 (10 calls)
Stamps returned to make 10 cents: 10 (11 calls)
Stamps returned to make 11 cents: 11 (12 calls)
Stamps returned to make 12 cents: 12 (13 calls)
Stamps returned to make 13 cents: 13 (14 calls)
Stamps returned to make 14 cents: 14 (15 calls)
Stamps returned to make 15 cents: 15 (16 calls)
Stamps returned to make 16 cents: 16 (17 calls)
Stamps returned to make 17 cents: 17 (18 calls)
Stamps returned to make 18 cents: 18 (19 calls)
Stamps returned to make 19 cents: 19 (20 calls)
Stamps returned to make 20 cents: 20 (21 calls)
Stamps returned to make 21 cents: 1 (23 calls)
Stamps returned to make 22 cents: 2 (26 calls)
Stamps returned to make 23 cents: 3 (30 calls)
Stamps returned to make 24 cents: 4 (35 calls)
Stamps returned to make 25 cents: 5 (41 calls)
Stamps returned to make 26 cents: 6 (48 calls)
Stamps returned to make 27 cents: 7 (56 calls)
Stamps returned to make 28 cents: 8 (65 calls)
Stamps returned to make 29 cents: 9 (75 calls)
Stamps returned to make 30 cents: 10 (86 calls)
Stamps returned to make 31 cents: 11 (98 calls)
Stamps returned to make 32 cents: 12 (111 calls)
Stamps returned to make 33 cents: 13 (125 calls)
Stamps returned to make 34 cents: 14 (140 calls)
Stamps returned to make 35 cents: 15 (156 calls)
Stamps returned to make 36 cents: 16 (173 calls)
Stamps returned to make 37 cents: 1 (192 calls)
Stamps returned to make 38 cents: 2 (213 calls)
Stamps returned to make 39 cents: 3 (236 calls)
Stamps returned to make 40 cents: 4 (261 calls)
Stamps returned to make 41 cents: 5 (288 calls)
Stamps returned to make 42 cents: 2 (318 calls)
Stamps returned to make 43 cents: 3 (352 calls)
Stamps returned to make 44 cents: 4 (391 calls)
Stamps returned to make 45 cents: 5 (436 calls)
Stamps returned to make 46 cents: 6 (488 calls)
Stamps returned to make 47 cents: 7 (548 calls)
Stamps returned to make 48 cents: 8 (617 calls)
Stamps returned to make 49 cents: 9 (696 calls)
Stamps returned to make 50 cents: 10 (786 calls)
Stamps returned to make 51 cents: 11 (888 calls)
Stamps returned to make 52 cents: 12 (1003 calls)
Stamps returned to make 53 cents: 13 (1132 calls)
Stamps returned to make 54 cents: 14 (1276 calls)
Stamps returned to make 55 cents: 15 (1436 calls)
Stamps returned to make 56 cents: 16 (1613 calls)
Stamps returned to make 57 cents: 17 (1808 calls)
Stamps returned to make 58 cents: 2 (2024 calls)
Stamps returned to make 59 cents: 3 (2264 calls)
Stamps returned to make 60 cents: 4 (2531 calls)
Stamps returned to make 61 cents: 5 (2828 calls)
Stamps returned to make 62 cents: 6 (3158 calls)
Stamps returned to make 63 cents: 3 (3525 calls)
Stamps returned to make 64 cents: 4 (3934 calls)
Stamps returned to make 65 cents: 5 (4391 calls)
Stamps returned to make 66 cents: 6 (4903 calls)
Stamps returned to make 67 cents: 7 (5478 calls)
Stamps returned to make 68 cents: 8 (6125 calls)
Stamps returned to make 69 cents: 9 (6854 calls)
Stamps returned to make 70 cents: 10 (7676 calls)
Stamps returned to make 71 cents: 11 (8603 calls)
Stamps returned to make 72 cents: 12 (9648 calls)
Stamps returned to make 73 cents: 13 (10825 calls)
Stamps returned to make 74 cents: 2 (12150 calls)
Stamps returned to make 75 cents: 3 (13640 calls)
Stamps returned to make 76 cents: 4 (15313 calls)
Stamps returned to make 77 cents: 5 (17188 calls)
Stamps returned to make 78 cents: 6 (19285 calls)
Stamps returned to make 79 cents: 3 (21628 calls)
Stamps returned to make 80 cents: 4 (24245 calls)
Stamps returned to make 81 cents: 5 (27168 calls)
Stamps returned to make 82 cents: 6 (30433 calls)
Stamps returned to make 83 cents: 7 (34080 calls)
Stamps returned to make 84 cents: 4 (38154 calls)
Stamps returned to make 85 cents: 5 (42706 calls)
Stamps returned to make 86 cents: 6 (47794 calls)
Stamps returned to make 87 cents: 7 (53484 calls)
Stamps returned to make 88 cents: 8 (59851 calls)
Stamps returned to make 89 cents: 9 (66980 calls)
Stamps returned to make 90 cents: 10 (74967 calls)
Stamps returned to make 91 cents: 11 (83920 calls)
Stamps returned to make 92 cents: 12 (93960 calls)
Stamps returned to make 93 cents: 13 (105222 calls)
Stamps returned to make 94 cents: 14 (117856 calls)
Stamps returned to make 95 cents: 3 (132031 calls)
Stamps returned to make 96 cents: 4 (147936 calls)
Stamps returned to make 97 cents: 5 (165781 calls)
Stamps returned to make 98 cents: 6 (185798 calls)
Stamps returned to make 99 cents: 7 (208242 calls)
*/
